#include "gbox_applet.h"
#include "gbox_mailbox.h"
#include "gbox_menu.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <utime.h>
#include <sys/stat.h>
#include <sys/types.h>

MailBoxList *mb_list;
GboxApplet  *applet;
MailBox     *exchange_mb;
MailBox     *default_box;
GboxMenu    *mailbox_menu;

gint        tooltip_height, tooltip_width;
GtkWidget   *tip_list, *scroll_win;

void
show_tooltip(GtkButton *button, gpointer user_data)
{
  GtkRequisition req;
  GtkWidget      *box;
  gint x, y;

  stop_timer();
  update_tooltip();
  set_mail_icon();
  start_timer();
  
  gtk_clist_set_shadow_type(GTK_CLIST(tip_list), GTK_SHADOW_ETCHED_OUT);
  gtk_clist_set_column_auto_resize (GTK_CLIST(tip_list), 0, TRUE);
  
  gtk_clist_set_selection_mode(GTK_CLIST(tip_list), GTK_SELECTION_SINGLE);
  gtk_clist_set_column_width(GTK_CLIST(tip_list), 0, 
			     gtk_clist_optimal_column_width(GTK_CLIST(tip_list), 0));
  gtk_clist_set_column_width(GTK_CLIST(tip_list), 1, 
			     gtk_clist_optimal_column_width(GTK_CLIST(tip_list), 1));
  
  if (!applet->show_tooltip) return;

  gtk_widget_get_child_requisition(GTK_WIDGET(applet->gbox_applet), &req);
  gdk_window_get_origin(applet->gbox_applet->window, &x, &y);
  
  applet->tip = gtk_window_new(GTK_WINDOW_POPUP);
  gtk_container_set_border_width(GTK_CONTAINER(applet->tip), 0);

  box = gtk_hbox_new(0, 0);
  
  gtk_widget_show(GTK_WIDGET(tip_list));
  
  gtk_box_pack_start(GTK_BOX(box), GTK_WIDGET(tip_list), TRUE, TRUE, 0);
  gtk_widget_show(GTK_WIDGET(box));
  
  gtk_container_add(GTK_CONTAINER(applet->tip), GTK_WIDGET(box));

  /*  gtk_widget_get_child_requisition(GTK_WIDGET(applet->tip), &req);
  printf("w: %d    h: %d\n", req.width, req.height);


  gdk_window_get_geometry(GTK_WIDGET(applet->tip), &ax, &ay, &aw, &ah, &ad);

  printf("x: %d y: %d w: %d h: %d d: %d\n", ax, ay, aw, ah, ad);*/

  switch (applet->pos_tooltip) {
  case 0: /* ABOVE */
    if ((x + tooltip_width) > gdk_screen_width())
      x = gdk_screen_width() - 2*12 - tooltip_width;

    y -= tooltip_height;
    break;
  case 1: /* LEFT */
    if ((y + tooltip_height) > gdk_screen_height())
      y = gdk_screen_height() - tooltip_height;
    
    x -= 2*14;
    x -= tooltip_width;
    break;
  case 2: /* BELOW */
    if ((x + tooltip_width) > gdk_screen_width())
      x = gdk_screen_width() - 2*12 - tooltip_width;

    y += req.height;
    break;
  case 3: /* RIGHT */
    if ((y + tooltip_height) > gdk_screen_height())
      y = gdk_screen_height() - tooltip_height;
    
    x += req.width;
  }
  
  gtk_widget_popup(GTK_WIDGET(applet->tip), x, y);
}

void
hide_tooltip(GtkButton *button, gpointer user_data)
{
  if (!applet->show_tooltip) return;
  gtk_widget_destroy(GTK_WIDGET(applet->tip));
  tip_list = NULL;
}

void
row_length(char **text)
{
  GtkStyle *style = gtk_widget_get_default_style();

  
  if (tooltip_width < (gdk_string_width(style->font, text[0]) + 
		       gdk_string_width(style->font, text[1]))) {
      tooltip_width = gdk_string_width(style->font, text[0])
	+ gdk_string_width(style->font, text[1]);
  }
  
}
  
void
update_tooltip()
{
  gchar     *text[2];
  GtkStyle  *style = gtk_widget_get_default_style();
  int       gap, baseline_skip, line_height;
  int       row_count = 0;
  MailBox   *walk = mb_list->first_mb;
  GdkColor  hilight_color = { 0, 0xCCDD, 0x1111, 0x1111 };
/*    GtkRequisition req; */
  
  tooltip_height = tooltip_width = 0;
  gap = (style->font->ascent + style->font->descent) / 4;
  if (gap < 2)
    gap = 2;
  
  line_height = gdk_string_height(style->font, "a") + style->font->ascent + style->font->descent;
  
  baseline_skip = style->font->ascent + style->font->descent + gap;
  
  if (!tip_list) tip_list = gtk_clist_new(2);
  gtk_clist_clear(GTK_CLIST(tip_list));
  
  switch (applet->style_tooltip) {
  case 0:
    text[0] = _("watched mailboxes");
    tooltip_height += line_height;
    text[1] = g_strdup_printf("%d", gbox_mailbox_list_count(mb_list));
    gtk_clist_append(GTK_CLIST(tip_list), text);
    row_length(text);
    
    g_free(text[1]);
    
    text[0] = _("contain new mail");
    tooltip_height += line_height;
    text[1] = g_strdup_printf("%d", gbox_mailbox_list_count_with_new_mail(mb_list));
    gtk_clist_append(GTK_CLIST(tip_list), text);

    row_length(text);

    g_free(text[1]);
    break;
  case 1:
    text[0] = _("highest priority");
    tooltip_height += line_height;
    text[1] = g_strdup_printf("%d", mb_list->new_count[0]);
    gtk_clist_append(GTK_CLIST(tip_list), text);
    row_length(text);
    
    g_free(text[1]);
    
    text[0] = _("middle priority");
    tooltip_height += line_height;
    text[1] = g_strdup_printf("%d", mb_list->new_count[1]);
    gtk_clist_append(GTK_CLIST(tip_list), text);
    row_length(text);
    
    g_free(text[1]);

    text[0] = _("lowest priority");
    tooltip_height += line_height;
    text[1] = g_strdup_printf("%d", mb_list->new_count[2]);
    gtk_clist_append(GTK_CLIST(tip_list), text);
    row_length(text);
    
    g_free(text[1]);
    break;
  case 2:
    while (walk) {
      text[0] = walk->nickname;
      tooltip_height += line_height;
      text[1] = g_strdup_printf("%d", walk->mail_count);
      
      gtk_clist_append(GTK_CLIST(tip_list), text);
      row_length(text);
      g_free(text[1]);
      
      if (walk->mail_count)
	gtk_clist_set_foreground(GTK_CLIST(tip_list), row_count, &hilight_color);
      
      walk = walk->next;row_count++;
    }
    break;
  case 3:
    while (walk) {
      if (walk->mail_count) {
	text[0] = walk->nickname;
	tooltip_height += line_height;
	text[1] = g_strdup_printf("%d", walk->mail_count);
	
	gtk_clist_append(GTK_CLIST(tip_list), text);
	row_length(text);
	
	g_free(text[1]);
	row_count++;
      }
      walk = walk->next;
    }
  }

  walk = mb_list->first_mb;
  while (walk) {
    gbox_menu_remove_box(mailbox_menu, walk);
    
    if (walk->show_in_menu)
      gbox_menu_add_box(mailbox_menu, walk);
      
    walk = walk->next;
  }
  
/*    gtk_widget_get_child_requisition(GTK_WIDGET(tip_list), &req); */

  tooltip_height += 12 + row_count;
  
  gtk_clist_set_row_height(GTK_CLIST(tip_list), line_height);
}

/*
  -3 - error
  -2 - size of mailbox became zero
  -1 - size of mailbox is less than before
   0 - size of mailbox is zero
   1 - size of mailbox is more than before
   2 - size of mailbox does not change but is not zero
   3 - new mail arrived in an empty mailbox
 */
int
check_mailbox(MailBox *mb)
{
  MailBox *walk = mb;
  struct stat buff;
  struct utimbuf timebuff;
  int    mail = 0;
  
  /*  if (!lstat(walk->mailfile, &buff)) { */
  if (!stat(walk->mailfile, &buff)) {
    if (buff.st_size == walk->size)
      mail = (!walk->size?0:2);
    else {
      if (walk->size < buff.st_size) {
	if (!walk->size) mail = 3;
	else mail = 1;
      } else {
	if (walk->size > buff.st_size) {
	  if (!buff.st_size) mail = -2;
	  else mail = -1;
	}
      }
    }
    
    if (buff.st_mtime > walk->modification_time
	|| buff.st_size != walk->size)
      walk->mail_count = gbox_mailbox_count_mails(walk->mailfile,
                                                           NULL, FALSE);
    
    walk->modification_time = buff.st_mtime;
    walk->size = buff.st_size;
  } else {
    mail = -3;
  }
  
  /* Make sure other programs checking for new mail (like mutt) are not
     affected by our check. This must come after
     gbox_mailbox_count_mails(). */
  timebuff.actime = buff.st_atime;
  timebuff.modtime = buff.st_mtime;
  utime(walk->mailfile, &timebuff);

  if (walk->changed_last_time)
    (mb_list->new_count[walk->priority])--;
  
  if (walk->mail_count > 0) {
    (mb_list->new_count[walk->priority])++;
    if (!walk->changed_last_time) {
      if (applet->style_beep)
	gdk_beep();
    }    
    walk->changed_last_time = TRUE;
  } else
    walk->changed_last_time = FALSE;
  
/*    switch (mail) { */
/*    case -2: */
/*      (mb_list->new_count[walk->priority])--; */
/*    	walk->changed_last_time = FALSE; */
/*      break; */
/*    case 3: */
/*      (mb_list->new_count[walk->priority])++; */
/*      if (applet->style_beep) */
/*        gdk_beep(); */
/*    	walk->changed_last_time = TRUE; */
/*    } */
  
  return mail;
}

void
update_mailboxes_cb(AppletWidget *app, gpointer data)
{
  MailBox * walk = mb_list->first_mb;
  
  while (walk) {
    check_mailbox(walk);
    walk = walk->next;
  }

  set_mail_icon();
}

void
set_mail_icon()
{
  int found_new = FALSE;
  int i;
  
  for (i = 0; i < COUNT_PRIORITY; i++)
    if (mb_list->new_count[i] > 0) {
      gtk_pixmap_set(GTK_PIXMAP(applet->image_win), 
		     applet->new_email_pixmap[i], 
		     applet->new_email_mask[i]);
      found_new = TRUE;
      break;
    }

  /*  update_tooltip();*/
  
  if (!found_new)
    gtk_pixmap_set(GTK_PIXMAP(applet->image_win), 
		   applet->no_email_pixmap, applet->no_email_mask);
}

gint
timeout_callback( gpointer data )
{
  check_mailbox((MailBox *) data);
  
  set_mail_icon();
  
  return 1; /* 0 will stop the timer forever */
}

static gint
exec_mail_prog(GtkWidget *widget, GdkEvent *evt, gpointer data)
{
    if (applet->mail_prog && (strlen(applet->mail_prog)) > 0)
	gnome_execute_shell(NULL, applet->mail_prog);

    return 1;
}

void
start_timer()
{
    MailBox *walk = mb_list->first_mb;
    
    while (walk) {
      	check_mailbox(walk);
	walk->timer = gtk_timeout_add( walk->interval,
				       timeout_callback,
				       walk);
	walk = walk->next;
    }
}

void
stop_timer()
{
    MailBox *walk = mb_list->first_mb;
    
    while (walk) {
	gtk_timeout_remove(walk->timer);
	walk = walk->next;
    }
}

static void
mailcheck_destroy (GtkWidget *widget, gpointer data)
{
    gtk_main_quit();
}

void
load_image(gchar *file, GdkImlibImage **image, GdkPixmap **pixmap, GdkBitmap **mask)
{
  GdkImlibImage *im;
  GdkPixmap     *mk, *px;

  im = *image;
  px = *pixmap;
  mk = *mask;
  
  *image = gdk_imlib_load_image(file);

  gdk_imlib_render(*image, 
		   (*image)->rgb_width, 
		   (*image)->rgb_height); 
  *pixmap = gdk_imlib_move_image(*image);
  *mask = gdk_imlib_move_mask(*image);

  g_free(im);
  /*  g_free(px);
      g_free(mk); */
}

static GtkWidget *
create_mail_widgets (GtkWidget *parent)
{
  GtkWidget *box;
  int       i;
  
  applet->no_email_image = NULL;
  applet->no_email_pixmap = NULL;
  applet->no_email_mask = NULL;
  
  box = gtk_hbox_new (0, 0);
  applet->main_button = gtk_button_new();
  gtk_signal_connect (GTK_OBJECT (box), "destroy",
		      (GtkSignalFunc) mailcheck_destroy,
		      NULL);
  
  gtk_widget_show (box);
  
  
  if (!g_file_test(applet->no_mail_pix, G_FILE_TEST_ISFILE)) {
    printf(_("Could not find the specified image-file (%s)\n"), applet->no_mail_pix);
    exit(1);
  }
    
  load_image(applet->no_mail_pix, 
	     &applet->no_email_image, &applet->no_email_pixmap, &applet->no_email_mask);
  
  for (i = 0; i < COUNT_PRIORITY; i++) {
    if ((applet->new_email_image[i] = gdk_imlib_load_image(applet->new_mail_pix[i])) == NULL) {
      printf(_("Ooops: please, do not give any non-existing image-file to me!!\n!"));
      exit(1);
    }
  
    gdk_imlib_render(applet->new_email_image[i], 
		     applet->new_email_image[i]->rgb_width, 
		     applet->new_email_image[i]->rgb_height);
    applet->new_email_pixmap[i] = gdk_imlib_move_image(applet->new_email_image[i]);
    applet->new_email_mask[i] = gdk_imlib_move_mask(applet->new_email_image[i]);
  }
  
  applet->image_win = gtk_pixmap_new( applet->no_email_pixmap, applet->no_email_mask );
  gtk_container_add(GTK_CONTAINER(applet->main_button), applet->image_win);
  
  if (applet->style_relief)
    gtk_button_set_relief(GTK_BUTTON(applet->main_button), GTK_RELIEF_NONE);
  
  gtk_widget_show(applet->main_button);
  gtk_signal_connect( GTK_OBJECT(applet->main_button), "clicked",
		      GTK_SIGNAL_FUNC(exec_mail_prog), 
		      NULL);
  
  gtk_box_pack_start(GTK_BOX(box), applet->main_button, FALSE, FALSE, 0);
  gtk_widget_show(applet->image_win);
  gtk_widget_show(box);
  
  return box;
}

static void
about_cb(AppletWidget* w, gpointer cbdata)
{
  GtkWidget* about_box, *l, *hbox;
  
  static const gchar* author[] = { N_("Andreas Bsching (crunchy@tzi.de) - programming"), 
				   N_("Thomas Buntrock (bunti@tzi.de) - graphics & programming"), 
				   NULL };
  

  about_box = gnome_about_new(_(PACKAGE), VERSION,
			      "(C) 1999 Andreas Bsching",
			      author,
			      _("Watching multiply mbox files ..."),
			      NULL);

  hbox = gtk_hbox_new (TRUE, 0);
  l = gnome_href_new ("http://www.informatik.uni-bremen.de/~crunchy/linux/gbox_applet.html",
		      _("Gbox Applet Home Page"));
  gtk_box_pack_start (GTK_BOX (hbox), l, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (about_box)->vbox),
		      hbox, TRUE, FALSE, 0);
  gtk_widget_show_all (hbox);
  
  gtk_widget_show(about_box);
}

static void
save_session_info(void)
{
  MailBox *walk = mb_list->first_mb;
  gchar box[128];
  int i;
  
  gnome_config_push_prefix(APPLET_WIDGET(applet->gbox_applet)->privcfgpath);
  
  gnome_config_set_string("/gbox_applet/global/no_mail_pixmap", applet->no_mail_pix);
  gnome_config_set_string("/gbox_applet/global/mail_prog", applet->mail_prog);
  gnome_config_set_bool("/gbox_applet/global/beep", applet->style_beep);
  gnome_config_set_bool("/gbox_applet/global/relief", applet->style_relief);
  gnome_config_set_bool("/gbox_applet/global/tooltip", applet->show_tooltip);
  gnome_config_set_int("/gbox_applet/global/tooltip_style", applet->style_tooltip);
  gnome_config_set_int("/gbox_applet/global/tooltip_pos", applet->pos_tooltip);
  gnome_config_set_int("/gbox_applet/global/count_mailbox", gbox_mailbox_list_count(mb_list));

  gnome_config_set_string("/gbox_applet/default_mailbox/filename", default_box->mailfile);
  gnome_config_set_string("/gbox_applet/default_mailbox/nickname", default_box->nickname);
  gnome_config_set_int("/gbox_applet/default_mailbox/interval", default_box->interval);
  gnome_config_set_int("/gbox_applet/default_mailbox/priority", default_box->priority);
  gnome_config_set_int("/gbox_applet/default_mailbox/menu_type", default_box->menu_type);
  gnome_config_set_bool("/gbox_applet/default_mailbox/show_in_menu", default_box->show_in_menu);

  for (i = 0;i < gbox_mailbox_list_count(mb_list); i++) {
    snprintf(box, sizeof(box), "/gbox_applet/mailbox%d/filename", i);
    gnome_config_set_string(box, walk->mailfile);
    snprintf(box, sizeof(box), "/gbox_applet/mailbox%d/nickname", i);
    gnome_config_set_string(box, walk->nickname);
    snprintf(box, sizeof(box), "/gbox_applet/mailbox%d/interval", i);
    gnome_config_set_int(box, walk->interval);
    snprintf(box, sizeof(box), "/gbox_applet/mailbox%d/priority", i);
    gnome_config_set_int(box, walk->priority);
    snprintf(box, sizeof(box), "/gbox_applet/mailbox%d/show_in_menu", i);
    gnome_config_set_bool(box, walk->show_in_menu);
    snprintf(box, sizeof(box), "/gbox_applet/mailbox%d/menu_type", i);
    gnome_config_set_int(box, walk->menu_type);
    walk = walk->next;
  }
  
  gnome_config_set_int("/gbox_applet/priority/count", COUNT_PRIORITY);
  for (i = 0;i < COUNT_PRIORITY; i++) {
    snprintf(box, sizeof(box), "/gbox_applet/priority/prio-%d", i);
    gnome_config_set_string(box, applet->new_mail_pix[i]);
  }
  
  gnome_config_pop_prefix();
  gnome_config_sync();
  gnome_config_drop_all();
}

static void
quit_applet(GtkWidget* w, gpointer cbdata)
{
  save_session_info();
  applet_widget_gtk_main_quit();
}

static gint
applet_save_session(GtkWidget* w, const gchar* priv, const gchar* global)
{
   save_session_info();
   return FALSE;
}

static void
read_config_data(gchar* cfgpath)
{
   gchar   *tmp, *nick, *help;
   int     count_mb = 0, i, num, prio, count_prio = 0, show, menu_type;
   char    box[128];
   char    pixmaps[1024];
   MailBox *mailbox;
   
   if(cfgpath == NULL)
	 return;
   gnome_config_push_prefix(cfgpath);
   
   snprintf(pixmaps, sizeof(pixmaps), 
	    "/gbox_applet/global/no_mail_pixmap=%s", 
	    gnome_pixmap_file("gbox_applet/default-no-mail.xpm"));
   
   printf("PIXMAP: %s\n", pixmaps);
   
   tmp = gnome_config_get_string(pixmaps);
   applet->no_mail_pix = g_strdup(tmp);
   g_free(tmp);

   tmp = gnome_config_get_string("/gbox_applet/global/mail_prog=emacs -f gnus");
   applet->mail_prog = g_strdup(tmp);
   g_free(tmp);

   applet->style_beep = gnome_config_get_bool("/gbox_applet/global/beep=false");
   applet->style_relief = gnome_config_get_bool("/gbox_applet/global/relief=false");
   applet->show_tooltip = gnome_config_get_bool("/gbox_applet/global/tooltip=false");   
   applet->style_tooltip = gnome_config_get_int("/gbox_applet/global/tooltip_style=0");
   applet->pos_tooltip = gnome_config_get_int("/gbox_applet/global/tooltip_pos=0");
   count_mb = gnome_config_get_int("/gbox_applet/global/count_mailbox=0");

   tmp = gnome_config_get_string("/gbox_applet/default_mailbox/filename");
   default_box->mailfile = g_strdup(tmp);
   g_free(tmp);
   tmp = gnome_config_get_string("/gbox_applet/default_mailbox/nickname");
   default_box->nickname = g_strdup(tmp);
   g_free(tmp);
   default_box->interval = gnome_config_get_int("/gbox_applet/default_mailbox/interval=0");
   default_box->show_in_menu = gnome_config_get_bool("/gbox_applet/default_mailbox/show_in_menu=false");
   default_box->menu_type = gnome_config_get_int("/gbox_applet/default_mailbox/menu_type=0");
   default_box->priority = gnome_config_get_int("/gbox_applet/default_mailbox/priority=2");

   for (i = 0; i < count_mb; i++) {
     snprintf(box, (size_t) sizeof(box), "/gbox_applet/mailbox%d/filename", i);
     tmp = gnome_config_get_string(box);
     snprintf(box, (size_t) sizeof(box), "/gbox_applet/mailbox%d/nickname", i);
     nick = gnome_config_get_string(box);
     if (!nick) {
       help = strrchr(tmp, '/');
       if (help) nick = help + 1;
     }
     
     snprintf(box, (size_t) sizeof(box), "/gbox_applet/mailbox%d/interval", i);
     num = gnome_config_get_int(box);
     snprintf(box, (size_t) sizeof(box), "/gbox_applet/mailbox%d/show_in_menu=false", i);
     show = gnome_config_get_bool(box);
     snprintf(box, (size_t) sizeof(box), "/gbox_applet/mailbox%d/menu_type=0", i);
     menu_type = gnome_config_get_int(box);
     snprintf(box, (size_t) sizeof(box), "/gbox_applet/mailbox%d/priority=%d", i, COUNT_PRIORITY - 1);
     prio = gnome_config_get_int(box);
     
     mailbox = gbox_mailbox_new(tmp, nick, num, prio, show, menu_type);
     
     gbox_mailbox_add(mb_list, mailbox);
   }

   snprintf(box, (size_t) sizeof(box), "/gbox_applet/priority/count=%d", COUNT_PRIORITY);
   count_prio = gnome_config_get_int(box);
   for (i = 0; i < count_prio; i++) {
     snprintf(box, (size_t) sizeof(box), "gbox_applet/default-prio-%d.xpm", i);
     snprintf(pixmaps, sizeof(pixmaps), 
	      "/gbox_applet/priority/prio-%d=%s", i, 
	      gnome_pixmap_file(box));
     applet->new_mail_pix[i] = gnome_config_get_string(pixmaps);
   }
   
   gnome_config_pop_prefix();
}

int
main(int argc, char *argv[])
{
  int i;

  applet = (GboxApplet *)malloc(sizeof(GboxApplet));
	applet->prop_box_active = FALSE;

  exchange_mb = (MailBox *)malloc(sizeof(MailBox));
  exchange_mb->mailfile = NULL;
  exchange_mb->nickname = NULL;

  default_box = (MailBox *)malloc(sizeof(MailBox));
  default_box->mailfile = NULL;
  default_box->nickname = NULL;

  mb_list = gbox_mailbox_list_new();
  
  mb_list->first_mb = NULL;
  mb_list->last_mb = NULL;
  tip_list = NULL;
  
  for (i = 0; i < COUNT_PRIORITY; i++)
      mb_list->new_count[i] = 0;
  
  bindtextdomain(PACKAGE, GNOMELOCALEDIR);
  textdomain(PACKAGE);
  
  applet_widget_init(PACKAGE, VERSION, argc, argv,
		     NULL, 0, NULL); 
  /* for using imlib */
  gdk_imlib_init();  
  gtk_widget_push_visual(gdk_imlib_get_visual());
  gtk_widget_push_colormap(gdk_imlib_get_colormap());
  /* end for using imlib */

  applet->gbox_applet = applet_widget_new(PACKAGE);
  
  if (!applet->gbox_applet) {
    printf("failed to create applet!\n");
    exit(0);
  }
  
  gtk_widget_realize(applet->gbox_applet);
  
  /* reading configuration */
  read_config_data(APPLET_WIDGET(applet->gbox_applet)->privcfgpath);
  
  applet->mailcheck = create_mail_widgets(applet->gbox_applet);
  gtk_widget_show(applet->mailcheck);
  applet_widget_add (APPLET_WIDGET (applet->gbox_applet), applet->mailcheck);
  
  gtk_widget_show(applet->gbox_applet);
  
  mailbox_menu = gbox_menu_new(APPLET_WIDGET(applet->gbox_applet));

  /* tooltip */
  start_timer();
  set_mail_icon();
  update_tooltip();  

  gtk_signal_connect(GTK_OBJECT(applet->gbox_applet), "delete_event",
		     GTK_SIGNAL_FUNC(quit_applet), NULL);
  gtk_signal_connect(GTK_OBJECT(applet->gbox_applet), "save_session",
		     GTK_SIGNAL_FUNC(applet_save_session), NULL);
  gtk_signal_connect(GTK_OBJECT(applet->main_button), "enter",
		     GTK_SIGNAL_FUNC(show_tooltip), NULL);
  gtk_signal_connect(GTK_OBJECT(applet->main_button), "leave",
		     GTK_SIGNAL_FUNC(hide_tooltip), NULL);

  applet_widget_register_stock_callback (APPLET_WIDGET(applet->gbox_applet),
					 "properties",
					 GNOME_STOCK_MENU_PROP,
					 _("properties"),
					 applet_properties,
					 NULL);

  applet_widget_register_stock_callback(APPLET_WIDGET(applet->gbox_applet),
					"update",
					GNOME_STOCK_MENU_REFRESH,
					_("update"),
					update_mailboxes_cb, NULL);

  applet_widget_register_stock_callback(APPLET_WIDGET(applet->gbox_applet),
					"about",
					GNOME_STOCK_MENU_ABOUT,
					_("About..."),
					about_cb, NULL);
  
  applet_widget_gtk_main(); 
  
  gbox_menu_free(mailbox_menu);
  gbox_mailbox_list_free(mb_list);

  free(mb_list);
  free(exchange_mb);
  free(default_box);
  free(applet);
  
  return 0;
}



